package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.Department;
import cn.efunbox.manage.base.enums.BaseStatusEnum;
import cn.efunbox.manage.base.repository.DepartmentRepository;
import cn.efunbox.manage.base.service.DepartmentService;
import cn.efunbox.manage.base.vo.DeptTreeVO;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * DepartmentServiceImpl
 * Created by wangys on 2019/03/07
 */
@Service
public class DepartmentServiceImpl implements DepartmentService {

    @Autowired
    DepartmentRepository departmentRepository;

    @Override
    public ApiResult getDeptTree(Long deptId) {
        return ApiResult.ok(recursiveTree(deptId));
    }

    @Override
    public ApiResult list(Department department) {
        List<Department> departments = departmentRepository.findByStatusOrderBySortDesc(BaseStatusEnum.NORMAL);
        if(Objects.nonNull(departments)){
            return ApiResult.ok(departments);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult save(Department department) {
        department = departmentRepository.save(department);
        if(Objects.nonNull(department)){
            return ApiResult.ok(department);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult update(Department department) {
        department = departmentRepository.update(department);
        if(Objects.nonNull(department)){
            return ApiResult.ok(department);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }


    @Override
    public ApiResult getDeptChild(Long deptId) {
        List<Department> departments = new ArrayList<>();
        departments = deptChild(deptId,departments);
        departments.add(departmentRepository.find(deptId));
        return ApiResult.ok(departments);
    }

    private DeptTreeVO recursiveTree(Long cid) {
        Department department = departmentRepository.find(cid);
        DeptTreeVO node = new DeptTreeVO();
        node.setLabel(department.getName());
        node.setCid(department.getId());
        List<Department> childTreeNodes = departmentRepository.findByPath(cid);
        for(Department child : childTreeNodes){
            DeptTreeVO n = recursiveTree(child.getId()); //递归
            node.getChildren().add(n);
        }
        return node;
    }

    private List<Department> deptChild(Long cid, List<Department> departments) {
        List<Department> childTreeNodes = departmentRepository.findByPath(cid);
        departments.addAll(childTreeNodes);
        for(Department child : childTreeNodes){
            deptChild(child.getId(), departments); //递归
        }
        return departments;
    }
}
