package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.Manage;
import cn.efunbox.manage.base.repository.ManageRepository;
import cn.efunbox.manage.base.service.ManageService;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * ManageSreviceImpl
 * Created by wangys on 2019/03/08
 */
@Service
public class ManageServiceImpl implements ManageService {

    @Autowired
    ManageRepository manageRepository;

    @Override
    public ApiResult list(Manage manage) {
        Specification querySpecifi = new Specification<Manage>() {
            @Override
            public Predicate toPredicate(Root<Manage> root, CriteriaQuery<?> criteriaQuery, CriteriaBuilder criteriaBuilder) {

                List<Predicate> predicates = new ArrayList<>();
                if(!StringUtils.equals("0", manage.getCompanyId()) && StringUtils.isNotBlank(manage.getCompanyId())){
                    predicates.add(criteriaBuilder.equal(root.get("companyId"), manage.getCompanyId()));
                }
                if(!StringUtils.equals("0", manage.getProductId()) && StringUtils.isNotBlank(manage.getProductId())){
                    predicates.add(criteriaBuilder.equal(root.get("productId"), manage.getProductId()));
                }
                if(!StringUtils.equals("0", manage.getProductId()) && StringUtils.isNotBlank(manage.getProductId())){
                    predicates.add(criteriaBuilder.equal(root.get("projectId"), manage.getProductId()));
                }
                if(Objects.nonNull(manage.getType())){
                    predicates.add(criteriaBuilder.equal(root.get("type"), manage.getType()));
                }
                if(!StringUtils.equals("0", manage.getCostType()) && StringUtils.isNotBlank(manage.getCostType())){
                    predicates.add(criteriaBuilder.equal(root.get("costType"), manage.getCostType()));
                }
                if(Objects.nonNull(manage.getIncomeStatus())){
                    predicates.add(criteriaBuilder.equal(root.get("incomeStatus"), manage.getIncomeStatus()));
                }
                if(manage.getDeptIds() != null && manage.getDeptIds().size() > 0){
                    List<Predicate> list = new ArrayList<>();
                    if (manage.getDeptIds() != null && manage.getDeptIds().size() > 0) {
                        CriteriaBuilder.In<Object> in = criteriaBuilder.in(root.get("deptId"));
                        for (Long id : manage.getDeptIds()) {
                            in.value(id);
                        }
                        list.add(in);
                    }
                    predicates.addAll(list);
                }
                return criteriaBuilder.and(predicates.toArray(new Predicate[predicates.size()]));
            }
        };
        List<Manage> manages = manageRepository.findAll(querySpecifi);
        if(Objects.nonNull(manages)){
            return ApiResult.ok(manages);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult save(Manage manage) {
        manage = manageRepository.save(manage);
        if(Objects.nonNull(manage)){
            return ApiResult.ok(manage);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult update(Manage manage) {
        manage = manageRepository.update(manage);
        if(Objects.nonNull(manage)){
            return ApiResult.ok(manage);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }
}
